/**
 * Created by lcg on 16/9/5.
 */

'use strict';

import { unlink } from 'react-native-omfs';
import CacheManager from './cacheManager';
import utils from './utils';
import { cacheDir } from './pathHelper';

class TaskInfo {
    constructor(taskId, type, cbDone, cbError) {
        this.taskId = taskId;
        this.type = type;
        this.cbDone = cbDone;
        this.cbError = cbError;
    }
}

export default class ClearManager {
    _clearTask = [];

    clear(taskId, type, cbDone, cbError){
        let clearTask = this._clearTask[type];
        if (clearTask) {
            clearTask.tasks[taskId] = new TaskInfo(taskId, type, cbDone, cbError);
            clearTask.count++;
        }
        else {
            clearTask = {
                tasks: {[taskId]: new TaskInfo(taskId, type, cbDone, cbError)},
                count: 1
            };

            this._clearTask[type] = clearTask;

            let self = this;

            CacheManager
                .removeType(type)
                .then(() =>{
                    unlink(cacheDir(type));
                })
                .then(() => {
                    self._onDone(type);
                })
                .catch((error) => {
                    self._onError(type, error);
                });
        };
    }

    cancel(type, taskId) {
        let clearTask = this._clearTask[type];
        if (!clearTask) {
            return;
        }

        clearTask.count--;

        if (clearTask.count <= 0) {
            clearTask.cancelled = true;
        }
        else {
            clearTask.tasks[taskId].cbError(new Error('cancelled'));
            delete clearTask.tasks[taskId];
        }
    }

    _onDone(type) {
        let clearTask = this._clearTask[type];
        if (clearTask) {
            for (let taskId in clearTask.tasks) {
                if (clearTask.tasks[taskId].cbDone) {
                    clearTask.tasks[taskId].cbDone();
                }
            }
            delete this._clearTask[type];
        }
    }

    _onError(type, error) {
        let clearTask = this._clearTask[type];
        if (clearTask) {
            for (let taskId in clearTask.tasks) {
                if (clearTask.tasks[taskId].cbError) {
                    clearTask.tasks[taskId].cbError(error);
                }
            }
            delete this._clearTask[type];
        }
    }
}