/**
 * Created by lcg on 16/9/4.
 */

import Requests, { SESSION_TYPE_DEFAULT } from 'react-native-requests';
import { unlink, mv, imageType } from 'react-native-omfs';
import { generatePath } from './pathHelper';
import CacheManager from './cacheManager';
import utils from './utils';
import { FileType } from './fileType';
import uuid from 'uuid';

class TaskInfo {
    constructor(taskId, uri, option, cbDone, cbError, cbCancel) {
        this.taskId = taskId;
        this.uri = uri;
        this.option = option;
        this.cbDone = cbDone;
        this.cbError = cbError;
    }
}

export default class FetchManager {
    constructor() {
        this._fetchTasks = {};
        this._requests = new Requests('file-cache', SESSION_TYPE_DEFAULT);
        this._requests.init();
    }

    fetch(taskId, uri, option, cbDone, cbError) {
        let filePath = '';
        let newPath = '';
        let fetchTask = this._fetchTasks[uri];
        if (fetchTask) {
            fetchTask.tasks[taskId] = new TaskInfo(taskId, uri, option, cbDone, cbError);
            fetchTask.count++;
        }
        else {
            fetchTask = {
                tasks: {[taskId]: new TaskInfo(taskId, uri, option, cbDone, cbError)},
                count: 1
            };

            this._fetchTasks[uri] = fetchTask;

            let self = this;

            CacheManager
                .getPath(uri)
                .then((path) => {
                    if (fetchTask.cancelled) {
                        throw new Error('cancelled');
                    }

                    if (utils.isString(path)) {
                        return path;
                    }

                    return generatePath(taskId, option.type)
                        .then((path) => {
                            if (fetchTask.cancelled) {
                                throw new Error('cancelled');
                            }

                            let config = {path};
                            if (utils.isString(option.ext)) {
                                config.appendExt = option.ext;
                            }

                            let method = option.method == null ? 'GET' : option.method;
                            let headers = option.headers || {};
                            let data = option.data;

                            fetchTask.path = path;
                            fetchTask.fetchObj = this._requests.createDownloadTask(uri, { path, headers, params: data });
                            fetchTask.fetchObj.cbProgress(({ current, total }) => {
                                self._onProgress(uri, current, total);
                            });

                            return new Promise((resolve, reject) => {
                                fetchTask.fetchObj.cbError(reject);
                                fetchTask.fetchObj.cbComplete(() => resolve(path));
                                fetchTask.fetchObj.cbCancelled(() => {
                                    fetchTask.cancelled = true;
                                    reject(new Error('cancelled'));
                                });
                                fetchTask.fetchObj.start();
                            });
                        })
                        .then((path) => {
                            filePath = path;
                            newPath = filePath;
                            return imageType(path)
                                .then(type => {
                                    newPath += '.' + type;
                                })
                                .catch(err => { /*not a image file*/});
                        })
                        .then(() => {
                            if(newPath != filePath){
                                return mv(filePath, newPath);
                            }
                        })
                        .then(() => {
                            return CacheManager.addPath(uri,newPath,option.type);
                        })
                })
                .then((path) => {
                    self._onDone(uri, path);
                })
                .catch((error) => {
                    self._onError(uri, error);
                });
        };
    }

    fetchImage(uri, option) {
        let buffer = new Array(32);
        uuid.v4(null, buffer, 0);
        let taskId = uuid.unparse(buffer);
        let self = this;

        option = option || {};
        option.type = option.type || FileType.Other;

        let promise = new Promise((resolve, reject) => {
            return self.fetch(taskId, uri, option, resolve, reject);
        });

        promise.cancel = () => {
            this._fetchManager.cancel(uri, taskId);
        };

        return promise;
    }

    cancel(uri, taskId) {
        let fetchTask = this._fetchTasks[uri];
        if (!fetchTask) {
            return;
        }

        fetchTask.count--;

        if (fetchTask.count <= 0) {
            fetchTask.cancelled = true;

            if (fetchTask.fetchObj) {
                unlink(fetchTask.path);
                fetchTask.fetchObj.cancel();
            }
        }
        else {
            fetchTask.tasks[taskId].cbError(new Error('cancelled'));
            delete fetchTask.tasks[taskId];
        }
    }

    _onProgress(uri, received, total) {
        let fetchTask = this._fetchTasks[uri];
        if (fetchTask) {
            for (let taskId in fetchTask.tasks) {
                if (fetchTask.tasks[taskId].option.onProgress) {
                    fetchTask.tasks[taskId].option.onProgress(received, total);
                }
            }
        }
    }

    _onDone(uri, path) {
        let fetchTask = this._fetchTasks[uri];
        if (fetchTask) {
            for (let taskId in fetchTask.tasks) {
                if (fetchTask.tasks[taskId].cbDone) {
                    fetchTask.tasks[taskId].cbDone(path);
                }
            }

            delete this._fetchTasks[uri];
        }
    }

    _onError(uri, error) {
        let fetchTask = this._fetchTasks[uri];
        if (fetchTask) {
            for (let taskId in fetchTask.tasks) {
                if (fetchTask.tasks[taskId].cbError) {
                    fetchTask.tasks[taskId].cbError(error);
                }
            }

            delete this._fetchTasks[uri];
        }
    }
}