/**
 * Created by lcg on 16/9/5.
 */

'use strict';

import { unlink } from 'react-native-omfs';
import CacheManager from './cacheManager';
import utils from './utils';

class TaskInfo {
    constructor(taskId, uri, cbDone, cbError) {
        this.taskId = taskId;
        this.uri = uri;
        this.cbDone = cbDone;
        this.cbError = cbError;
    }
}

export default class RemoveManager {
    _removeTask = [];

    remove(taskId, uri, cbDone, cbError){
        let removeTask = this._removeTask[uri];
        if (removeTask) {
            removeTask.tasks[taskId] = new TaskInfo(taskId, uri, cbDone, cbError);
            removeTask.count++;
        }
        else {
            removeTask = {
                tasks: {[taskId]: new TaskInfo(taskId, uri, cbDone, cbError)},
                count: 1
            };

            this._removeTask[uri] = removeTask;

            let self = this;

            CacheManager
                .getPath(uri)
                .then((path) => {
                    if (removeTask.cancelled) {
                        throw new Error('cancelled');
                    }

                    if (!utils.isString(path)) {
                        return null;
                    }

                    return unlink(path);
                })
                .then(() => {
                    self._onDone(uri);
                })
                .catch((error) => {
                    self._onError(uri, error);
                });
        };
    }

    cancel(uri, taskId) {
        let removeTask = this._removeTask[uri];
        if (!removeTask) {
            return;
        }

        removeTask.count--;

        if (removeTask.count <= 0) {
            removeTask.cancelled = true;
        }
        else {
            removeTask.tasks[taskId].cbError(new Error('cancelled'));
            delete removeTask.tasks[taskId];
        }
    }

    _onDone(uri) {
        let saveTask = this._removeTask[uri];
        if (saveTask) {
            for (let taskId in saveTask.tasks) {
                if (saveTask.tasks[taskId].cbDone) {
                    saveTask.tasks[taskId].cbDone();
                }
            }
            delete this._removeTask[uri];
        }
    }

    _onError(uri, error) {
        let saveTask = this._removeTask[uri];
        if (saveTask) {
            for (let taskId in saveTask.tasks) {
                if (saveTask.tasks[taskId].cbError) {
                    saveTask.tasks[taskId].cbError(error);
                }
            }
            delete this._removeTask[uri];
        }
    }
}