/**
 * Created by lcg on 16/9/4.
 */

import { cp } from 'react-native-omfs';
import { generatePath } from './pathHelper';
import CacheManager from './cacheManager';
import utils from './utils';

class TaskInfo {
    constructor(taskId, uri, source, option, cbDone, cbError) {
        this.taskId = taskId;
        this.uri = uri;
        this.source = source;
        this.option = option;
        this.cbDone = cbDone;
        this.cbError = cbError;
    }
}

export default class SaveManager {
    _saveTask = {};

    save(taskId, uri, source, option, cbDone, cbError) {
        let saveTask = this._saveTask[uri];
        if (saveTask) {
            saveTask.tasks[taskId] = new TaskInfo(taskId, uri, source, option, cbDone, cbError);
            saveTask.count++;
        }
        else {
            saveTask = {
                tasks: {[taskId]: new TaskInfo(taskId, uri, source, option, cbDone, cbError)},
                count: 1
            };

            this._saveTask[uri] = saveTask;

            let self = this;

            CacheManager
                .getPath(uri)
                .then((path) => {
                    if (saveTask.cancelled) {
                        throw new Error('cancelled');
                    }
                    if (utils.isString(path)) {
                        return path;
                    }

                    return generatePath(taskId, option.type)
                        .then((path) => {
                            if (saveTask.cancelled) {
                                throw new Error('cancelled');
                            }

                            let config = {path};
                            if (utils.isString(option.ext)) {
                                config.appendExt = option.ext;
                            }

                            let filename = path;
                            if (utils.isString(option.ext)) {
                                filename += '.' + option.ext;
                            }

                            return cp(source, filename)
                                .then(() => {
                                    return CacheManager.addPath(uri, filename, option.type);
                                })
                        });
                })
                .then((path) => {
                    self._onDone(uri, path);
                })
                .catch((error) => {
                    self._onError(uri, error);
                });
        };
    }

    cancel(uri, taskId) {
        let saveTask = this._saveTask[uri];
        if (!saveTask) {
            return;
        }

        saveTask.count--;

        if (saveTask.count <= 0) {
            saveTask.cancelled = true;
        }
        else {
            saveTask.tasks[taskId].cbError(new Error('cancelled'));
            delete saveTask.tasks[taskId];
        }
    }

    _onDone(uri, path) {
        let saveTask = this._saveTask[uri];
        if (saveTask) {
            for (let taskId in saveTask.tasks) {
                if (saveTask.tasks[taskId].cbDone) {
                    saveTask.tasks[taskId].cbDone(path);
                }
            }
            delete this._saveTask[uri];
        }
    }

    _onError(uri, error) {
        let saveTask = this._saveTask[uri];
        if (saveTask) {
            for (let taskId in saveTask.tasks) {
                if (saveTask.tasks[taskId].cbError) {
                    saveTask.tasks[taskId].cbError(error);
                }
            }
            delete this._saveTask[uri];
        }
    }
}