'use strict';
import React, { Component } from 'react';
import {
    Image,
    View,
    StyleSheet,
    TouchableOpacity,
    Platform
} from 'react-native';
import { FileCache, FileType } from '../filecache';
import utils from '../lib/utils';

export default class CacheImage extends Component {
    constructor(props) {
        super(props);
        this.state = {
            uri : this.props.source,
            source : null
        };
    }

    static getSource(uri){
        return fetchImage(uri);
    }

    componentDidMount(){
        let uri = this.state.uri;
        fetchImage(uri)
          .then((source) => {
              this.setState({source});
          });
    }

    componentWillReceiveProps(nextProps) {
        if (this.props.source !== nextProps.source) {
            this.setState({
                uri: nextProps.source,
                source : null
            });

            fetchImage(nextProps.source)
                .then((source) => {
                    this.setState({source});
                });
        }
    }

    render() {

        if(this.state.source){
            let source = this.state.source;
            let mainImage = (
                <Image style={[styles.img ,this.props.style]}
                       resizeMode={this.props.resizeMode || 'contain'}
                       source={typeof source == 'string' ? {uri : source} : source}/>
            );
            return mainImage;
        }else{
            return (
                <View style={[styles.img, this.props.style]}></View>
            )
        }
    }
}

function fetchImage(uri) {
    if(typeof uri == 'string'){
        if(utils.startWith(uri,'http://') || utils.startWith(uri,'https://')){//判断是否是网络图片 是则下载,否则直接展示
            return FileCache.fetch(uri,{type: FileType.Image}).then(function(path){
                if(utils.endWith(path,'.jpg') || utils.endWith(path,'.png') || utils.endWith(path,'.gif')){
                    return (Platform.OS === 'ios') ? path : 'file://' + path;
                }else{//未识别的图片类型,则直接展示网络图片
                    return uri;
                }
            }).catch(function(e){//获取本地路径异常 直接展示网络图片
                console.log('获取远程图片异常',e);
                return uri;
            });
        }else{//非网络图片 直接展示
            return Promise.resolve(uri);
        }
    }else{
        return Promise.resolve(uri);
    }
}

const styles = StyleSheet.create({
    img: {
        width: 150,
        height: 150,
        //borderRadius: 75,
    },
});